import binascii
import enum
from .signature import pedersen_hash


class InstructionType(enum.Enum):
    Transfer = 1
    Withdraw = 2
    Offer = 3
    Offer_cancel = 4


def get_msg(
        instruction_type: int, amount: int, fee: int,
        nonce: int, receiver: int, message: int) -> int:
    """
    Creates a message to sign on.
    """
    packed_message = instruction_type
    packed_message = packed_message * 2 ** 63 + amount

    if fee:
        packed_message = packed_message * 2 ** 63 + fee

    packed_message = packed_message * 2 ** 31 + nonce

    if receiver:
        tmp_hash = pedersen_hash(pedersen_hash(message), receiver)
    else:
        tmp_hash = pedersen_hash(message)
    return pedersen_hash(tmp_hash, packed_message)


def get_transfer_msg(action: dict, nonce: int, message: str) -> str:
    receiver = action['receiver']
    amount = action['amount']
    name = action['name']

    assert name == 'transfer'

    return hex(get_msg(InstructionType.Transfer.value, amount, 0, nonce, int(receiver, 16), message_str_to_int(message)))


def get_withdraw_msg(action: dict, nonce: int, message: str) -> str:
    fee = action['fee']
    amount = action['amount']
    name = action['name']
    assert name == 'withdraw'
    return hex(get_msg(InstructionType.Withdraw.value, amount, fee, nonce, 0, message_str_to_int(message)))


def get_offer_msg(action: dict, nonce: int, message: str) -> str:
    receiver = action['receiver']
    amount = action['amount']
    name = action['name']
    fee = action['fee']
    assert name == 'offer'

    return hex(get_msg(InstructionType.Offer.value, amount, fee, nonce, int(receiver, 16), message_str_to_int(message)))


def get_offer_cancel_msg(action: dict, nonce: int, message: str) -> str:
    receiver = action['receiver']
    amount = action['amount']
    name = action['name']
    assert name == 'offer_cancel'

    return hex(get_msg(InstructionType.Offer_cancel.value, amount, 0, nonce, int(receiver, 16), message_str_to_int(message)))


def message_str_to_int(message: str) -> int:
    if not message:
        return 0
    return int(f"0x{binascii.hexlify(message.encode('utf-8')).decode('utf-8')}", 16)


def message_int_to_str(message: int) -> str:
    if not message:
        return ''
    return binascii.unhexlify(hex(message)[2:].encode('utf-8')).decode('utf-8')
